using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GalaSoft.MvvmLight;
using System.Collections.ObjectModel;
using System.Collections.Generic;

namespace Benday.HyperlinkButtonSample.Presentation
{
    public class PersonViewModel : ViewModelBase
    {
        public void Save()
        {
            if (PersonInstance == null)
            {
                m_personInstance = new Person();
            }

            PersonInstance.FirstName = this.FirstName;
            PersonInstance.LastName = this.LastName;
            PersonInstance.EmailAddress = this.EmailAddress;

            PersonRepository.Instance.Save(PersonInstance, 
                new ReturnResult<Person>(result =>
                {
                    if (result.Result != null) Id = result.Result.Id;

                    if (NavigateHome != null) NavigateHome(this, new EventArgs());
                }));
        }

        public void LoadById(int id)
        {
            if (id == -1)
            {
                InitializeBlank();
            }
            else
            {
                PersonRepository.Instance.GetById(id, new ReturnResult<Person>(LoadByIdCompleted));    
            }            
        }

        private void InitializeBlank()
        {
            Id = -1;
            FirstName = String.Empty;
            LastName = String.Empty;
            EmailAddress = String.Empty;
        }

        private ICommand m_saveCommand;
        public ICommand SaveCommand
        {
            get
            {
                if (m_saveCommand == null)
                {
                    m_saveCommand = new SavePersonCommand(this);
                }

                return m_saveCommand;
            }
            set
            {
                m_saveCommand = value;
            }
        }

        private ICommand m_cancelCommand;
        public ICommand CancelCommand
        {
            get
            {
                if (m_cancelCommand == null)
                {
                    m_cancelCommand = new CancelPersonCommand(this);
                }

                return m_cancelCommand;
            }
            set
            {
                m_cancelCommand = value;
            }
        }

        public void Cancel()
        {
            if (NavigateHome != null) NavigateHome(this, new EventArgs());
        }

        public event EventHandler NavigateHome;

        private void LoadByIdCompleted(ReturnResult<Person> result)
        {
            if (result == null || result.Result == null)
            {
                throw new InvalidOperationException("Argument cannot be null.");
            }
            else if (result.Error != null)
            {
                throw result.Error;
            }
            else
            {
                PopulateFromModel(result.Result);
            }
        }

        private Person m_personInstance;
        private Person PersonInstance
        {
            get { return m_personInstance; }
        }

        private void PopulateFromModel(Person from)
        {
            m_personInstance = from;
            Id = from.Id;
            FirstName = from.FirstName;
            LastName = from.LastName;
            EmailAddress = from.EmailAddress;
        }

        private const string IdPropertyName = "Id";

        private int m_id;
        public int Id
        {
            get
            {
                return m_id;
            }
            set
            {
                m_id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }

        private const string FirstNamePropertyName = "FirstName";

        private string m_firstName;
        public string FirstName
        {
            get
            {
                return m_firstName;
            }
            set
            {
                m_firstName = value;
                RaisePropertyChanged(FirstNamePropertyName);
            }
        }

        private const string LastNamePropertyName = "LastName";

        private string m_lastName;
        public string LastName
        {
            get
            {
                return m_lastName;
            }
            set
            {
                m_lastName = value;
                RaisePropertyChanged(LastNamePropertyName);
            }
        }

        private const string EmailAddressPropertyName = "EmailAddress";

        private string m_emailAddress;
        public string EmailAddress
        {
            get
            {
                return m_emailAddress;
            }
            set
            {
                m_emailAddress = value;
                RaisePropertyChanged(EmailAddressPropertyName);
            }
        }
    }
}
