using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GalaSoft.MvvmLight;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Linq;

namespace Benday.HyperlinkButtonSample.Presentation
{
    public class RepositoryMock<T> : IRepository<T> where T : IInt32Identity
    {
        private int m_currentIdValue = 0;

        public RepositoryMock()
        {
            Items = new List<T>();
        }

        protected List<T> Items { get; set; }

        /// <summary>
        /// Add an item to the mock collection.  Note: this method is not part of IRepository&lt;T&gt;.
        /// </summary>
        /// <param name="item"></param>
        protected void Add(T item)
        {
            Items.Add(item);
        }

        public void Save(T saveThis, ReturnResult<T> action)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            if (saveThis.Id == 0)
            {
                saveThis.Id = ++m_currentIdValue;
            }

            if (Items.Contains(saveThis) == false)
            {
                Items.Add(saveThis);
            }

            action.Notify(saveThis);
        }

        protected virtual T BeforeReturn(T item)
        {
            return item;
        }

        protected virtual IList<T> BeforeReturn(IList<T> items)
        {
            return items;
        }

        public void GetById(int id, ReturnResult<T> action)
        {
            var result = BeforeReturn((from item in Items
                                       where item.Id == id
                                       select item).FirstOrDefault());

            action.Notify(result);
        }

        public void GetAll(ReturnResult<IList<T>> action)
        {
            var result = BeforeReturn(Items);

            action.Notify(result);
        }

        public void Delete(T deleteThis, ReturnResult<T> action)
        {
            Items.Remove(deleteThis);
            action.Notify(deleteThis);
        }
    }
}
